unit CommIni;

interface

uses Objects, AdvUtil, CommType, TCIGlobl,
{$IFDEF DPMI}
     IniImport
{$ELSE}
     ReadIni
{$ENDIF};

const ChanSectionPrefix: FNameStr = 'CHANNEL#';
      ObjSectionPrefix: FNameStr = 'OBJECT#';
      IniFileName  : FNameStr = 'COMMUNIC.INI';

function BeginCommIni(const vIniFileName: FNameStr) : boolean;
  {$IFDEF DPMI} export; {$ENDIF}
procedure EndCommIni;
  {$IFDEF DPMI} export; {$ENDIF}
function ReadChannelCfg(var ChanInit : TChannelInit; Number : byte) : boolean;
  {$IFDEF DPMI} export; {$ENDIF}
procedure InitGetObjectCfg;
  {$IFDEF DPMI} export; {$ENDIF}
function ReadObjectCfg(var ObjInit : TObjectCfg) : boolean;
  {$IFDEF DPMI} export; {$ENDIF}
function ReadMainCfg(var MainInit : TMainCfg) : boolean;
  {$IFDEF DPMI} export; {$ENDIF}
function InitCommIni : boolean;
procedure DoneCommIni;

implementation

const Opened : boolean = false;

function InitCommIni : boolean;
  begin
    InitCommIni := true;
  end;

procedure DoneCommIni;
  begin
    EndCommIni;
  end;

function BeginCommIni(const vIniFileName: FNameStr) : boolean;
  begin
    EndCommIni;
    if vIniFileName   <> '' then IniFileName   := vIniFileName;
    Opened := OpenIniFile(IniFileName);
    if not Opened then CloseIniFile;
    BeginCommIni := Opened;
  end;

procedure EndCommIni;
  begin
    if Opened then CloseIniFile;
    Opened := false;
  end;

const
  FlagStatement = 'TIMEADJUST ALTCONNECT WAITBEEP' +
  ' FLATCHERCHECK CRC32CHECK BAUDADJUST FRAMEADJUST CHECKBUSY' +
  ' MASTERMODE FIXEDMODE PHONEMODE STONEAGE ANSWERMODE';

function ReadChannelCfg(var ChanInit : TChannelInit; Number : byte) : boolean;
  var S : string;
      I : word;
  const ChanStatement = 'BASE IRQ PARAMS VECTOR INCOMINGCALL BAUDRATE' +
    ' BAUDRANGE RCBUFSIZE TFBUFSIZE MODEMTYPE LINE PROTOCOLMODE PROTOCOLTYPE'+
    ' CHECKSUMM';
    ProtocolStatement = 'ECOSYS-1 ECOSYS-2 EMSI';
    ModeStatement = 'REVERSE NORMAL SLAVE MASTER';
    HayesStatement = 'FLOWCONTROL REDIAL HARDWARERING INITMODEM BUFSIZE' +
    ' INIT1STRING INIT2STRING DIALPREFIX DIALSUFFIX ONHOOK OFFHOOK'+
    ' ANSWERSTRING CONNECT MODEMOK DIALTERMINATE STATISTICS BUSY'+
    ' NODIALTONE RING RINGING ATTEMPTON ANSWERON';
    ZModemStatement = '';
    FlowStatement = 'NO DSR CTS SOFT';
    ModemStatement = 'HAYES NULL MASTAK K1-MASTAK';
    LineStatement = 'SEPARATE PHONE CORDLESSPHONE';
    CheckSummStatement = 'CRC16 CRC32 FLATCHER';
  begin
    if SearchSection(ChanSectionPrefix + ExpIntStr(Number, 2)) then
      begin
        fillchar(ChanInit, sizeof(ChanInit), 0);
        ChanInit.BaudRange.Lo := 115200 div 110;
        ChanInit.BaudRange.Hi := 1;
        ChanInit.BaudRate := 115200 div 1200;
        ChanInit.FrameSize.Lo := 128;
        ChanInit.FrameSize.Hi := 128;
        while GetKey(S) do
          case CheckStatement(S, ChanStatement) of
            0 : begin
                  I := CheckStatement(S, FlagStatement);
                  case I of
                    0 : case CheckStatement(S, HayesStatement) of
                          0 : continue;
                          1 : begin
                                case GetStatementNumber(FlowStatement, 0) of
                                  0 : ChanInit.HayesWord.FlowControl := fcNone;
                                  1 : ChanInit.HayesWord.FlowControl := fcHardWareDSR;
                                  2 : ChanInit.HayesWord.FlowControl := fcHardWareCTS;
                                  3 : ChanInit.HayesWord.FlowControl := fcSoftWare;
                                end;
                              end;
                          2 : ChanInit.HayesWord.Redial := GetInteger(0, 30, 3);
                          3 : ChanInit.HayesWord.RingIndicator := GetBoolean(false);
                          4 : ChanInit.HayesWord.InitModem := GetBoolean(false);
                          5 : ChanInit.HayesWord.BufSize := GetInteger(128, 65520, 1024);
                          6 : ChanInit.HayesStr.Init1String := GetQuoteString('');
                          7 : ChanInit.HayesStr.Init2String := GetQuoteString('');
                          8 : ChanInit.HayesStr.DialPrefix := GetQuoteString('');
                          9 : ChanInit.HayesStr.DialSuffix := GetQuoteString('');
                          10: ChanInit.HayesStr.OnHook := GetQuoteString('');
                          11: ChanInit.HayesStr.OffHook := GetQuoteString('');
                          12: ChanInit.HayesStr.AnswerString := GetQuoteString('');
                          13: ChanInit.HayesStr.ConnectString := GetQuoteString('');
                          14: ChanInit.HayesStr.ModemOK := GetQuoteString('');
                          15: ChanInit.HayesStr.DialTerminate := GetQuoteString('');
                          16: ChanInit.HayesStr.Statistics := GetQuoteString('');
                          17: ChanInit.HayesStr.Busy := UpCaseStr(GetQuoteString(''));
                          18: ChanInit.HayesStr.NoDialtone := UpCaseStr(GetQuoteString(''));
                          19: ChanInit.HayesStr.Ring := UpCaseStr(GetQuoteString(''));
                          20: ChanInit.HayesStr.Ringing := UpCaseStr(GetQuoteString(''));
                          21: ChanInit.HayesStr.AttemptOn := UpCaseStr(GetQuoteString(''));
                          22: ChanInit.HayesStr.AnswerOn := UpCaseStr(GetQuoteString(''));
                        end;
                  else
                    i := 1 shl pred(i);
                    if GetBoolean(false) then
                      ChanInit.ModemFlags := ChanInit.ModemFlags or i
                    else ChanInit.ModemFlags := ChanInit.ModemFlags and not i;
                  end;
                end;
            1 : ChanInit.Base := GetInteger(0, $FFFF, 0);
            2 : ChanInit.Irq := GetInteger(2, 15, 3);
            3 : begin
                  S := ReadWord;
                  if length(S) < 3 then S := '8N1';
                  case S[1] of
                    '5' : ChanInit.Params := lc5bit;
                    '6' : ChanInit.Params := lc6bit;
                    '7' : ChanInit.Params := lc7bit;
                    '8' : ChanInit.Params := lc8bit;
                  end;
                  case upcase(S[2]) of
                    'N' : inc(ChanInit.Params, lcNoParity);
                    'E' : inc(ChanInit.Params, lcEven);
                    'O' : inc(ChanInit.Params, lcOdd);
                    'F' : inc(ChanInit.Params, lcFixParity);
                  end;
                  case S[3] of
                    '1' : inc(ChanInit.Params, lcStop1);
                    '2' : inc(ChanInit.Params, lcStop2);
                  end;
                end;
            4 : ChanInit.Vector := GetInteger(0, $FFFF, $2BF);
            5 : ChanInit.IncomingCall := GetBoolean(false);
            6 : ChanInit.BaudRate := 115200 div GetInteger(110, 115200, 1200);
            7 : begin
                  ChanInit.BaudRange.Lo := 115200 div GetInteger(110, 115200, 1200);
                  ChanInit.BaudRange.Hi := 115200 div GetInteger(110, 115200, 1200);
                end;
            8 : ChanInit.FrameSize.Lo := GetInteger(6, 65535, 1024);
            9 : ChanInit.FrameSize.Hi := GetInteger(6, 65535, 1024);
            10: ChanInit.Modem := GetStatementNumber(ModemStatement, 0);
            11: begin
                  ChanInit.Line := GetStatementNumber(LineStatement, 0);
                  if ChanInit.Line = 1 then
                    ChanInit.ModemFlags := ChanInit.ModemFlags or wfPhoneLine
                  else ChanInit.ModemFlags := ChanInit.ModemFlags and not wfPhoneLine;
                end;
            12: begin
                  I := GetStatementNumber(ModeStatement, 0);
                  case I of
                    0 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           and Not(wfMasterMode+wfFixedMode);
                    1 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           or wfMasterMode and (Not wfFixedMode);
                    2 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           and (Not wfMasterMode) or wfFixedMode;
                    3 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           or (wfMasterMode+wfFixedMode);
                  end;
                end;
            13: ChanInit.Protocol := GetStatementNumber(ProtocolStatement, 0);
            14: begin
                  case GetStatementNumber(CheckSummStatement, 0) of
                    0 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           and Not(wfFlatcherCheck+wfCrc32Check);
                    1 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           or wfCrc32Check and (Not wfFlatcherCheck);
                    2 : ChanInit.ModemFlags := ChanInit.ModemFlags
                           or wfFlatcherCheck and (Not wfCrc32Check);
                  end;
                end;
          end;
        ReadChannelCfg := true;
      end
    else ReadChannelCfg := false;
  end;

procedure InitGetObjectCfg;
  begin
    InitGetSection;
  end;

function ReadObjectCfg(var ObjInit : TObjectCfg) : boolean;
  var S : string;
      I : word;
  const
    ObjStatement = 'CHANNEL BAUDRATE HOTTIME CURFACTOR SUMFACTOR ARRAYFACTOR';
  begin
    ReadObjectCfg := false;
    if GetSection(S) and
      (copy(S, 1, length(ObjSectionPrefix)) = ObjSectionPrefix) then
      begin
        fillchar(ObjInit, sizeof(ObjInit), 0);
        val(copy(s, length(ObjSectionPrefix),
            length(s) - length(ObjSectionPrefix)), ObjInit.Address.SP, i);
        if i <> 0 then exit;
        while GetKey(S) do
          case CheckStatement(S, ObjStatement) of
            0 : begin
                  I := CheckStatement(S, FlagStatement);
                  if i <> 0 then
                    begin
                      i := 1 shl pred(i);
                      if GetBoolean(false) then
                        ObjInit.Options := ObjInit.Options or i
                      else ObjInit.Options := ObjInit.Options and not i;
                    end;
                end;
            1 : ObjInit.Channel := GetInteger(1, 16, 1);
            2 : ObjInit.Baud := 115200 div GetInteger(110, 115200, 1200);
            3 : ObjInit.HotTime := GetInteger(1, 3600, 30);
            4 : ObjInit.CurFactor := GetInteger(0, 10000, 3);
            5 : ObjInit.SumFactor := GetInteger(0, 10000, 10);
            6 : ObjInit.ArrFactor := GetInteger(0, 10000, 20);
          end;
        ReadObjectCfg := true;
      end;
  end;

function ReadMainCfg(var MainInit : TMainCfg) : boolean;
  var S : string;
      I : word;
  const
    MainStatement = 'NETZONE NETSYSTEM';
  begin
    if SearchSection('MAIN') then
      begin
        while GetKey(S) do
          case CheckStatement(S, MainStatement) of
            1 : MainInit.NetZone := GetInteger(0, 65535, MainInit.NetZone);
            2 : MainInit.NetSystem := GetInteger(0, 65535, MainInit.NetSystem);
          end;
        ReadMainCfg := true;
      end
    else ReadMainCfg := false;
  end;

end.
