Unit TCIGlobl;
Interface
Uses Objects;

Type
  TTciHandle = Byte;
  TChannelsSet = Set Of TTciHandle;

  TEcoSysAddress = record
    EcoSysFlag : Word;
    AskNet     : Word;
    Station    : Word;
    Module     : Word
  end;

  AddressStr  = String[23];
  PasswordStr = String[60];

  PEMSIConnectRec = ^TEMSIConnectRec;
  TEMSIConnectRec = record
    DestAddr    : AddressStr;
    Password    : PasswordStr;
    SrcAddrList : String
  end;

  PEmsiDat = ^TEmsiDat;
  TEmsiDat = record
    AddrList            : String;
    Password            : PasswordStr;
    Link_codes          : String[30];
    Compatibility_codes : String;
    Mailer_product_code : String[10];
    Mailer_name         : String[60];
    Mailer_version      : String[30];
    Mailer_serial_number: String[60];
    {Extra_field}
    ProductID           : String[20];
    System_name         : String;
    City                : String[40];
    Operator_name       : String[60];
    Phone_number        : String[60];
    Baud_rate           : String[7];
    Flags               : String
  end;

  TOpTypes = (opService, opReceive, opTransfer);
  TState   = (Nothing, Busy, Finished);

  TProtocolStatusRec = record
    RcvLen       : Word;
    SndLen       : Word;
    RcvFrmLen    : Word;
    SndFrmLen    : Word;
    RtrErrCount  : Word;
    CrcErrCount  : Word;
    TypeRcFrame  : Byte;
    TypeTfFrame  : Byte;
    CntRcFrame   : Byte;
    CntTfFrame   : Byte
  end;

  TStatusRec = record
    Complete  : Boolean;
    Try       : Byte;
    Operation : Byte;
    TCIError  : Byte;
    LostByte  : Byte;
    Receiver  : TState;
    Sender    : TState;
    Protocol  : TProtocolStatusRec
  end;

  PPhoneStr = ^TPhoneStr;
  TPhoneStr = String[40];

  PHayesStringRec = ^THayesStringRec;
  THayesStringRec = record
    Init1String  : String[127];  {String for init hayes modem}
    Init2String  : String[127];  {Added string for init hayes modem}
    DialPrefix   : String[030];  {Prefix for dialing}
    DialSuffix   : String[030];  {Suffix for dialing}
    OnHook       : String[030];  {Disconnect and go on-hook}
    OffHook      : String[030];  {Go off-hook}
    AnswerString : String[030];  {Go off-hook immediately}
    ConnectString: String[030];  {'CONNECT XXXX' response}
    ModemOK      : String[010];  {Force modem to answer 'OK'}
    DialTerminate: String[030];  {Terminate dial and go on-hook}
    Statistics   : String[030];  {Ask statistic}
    Busy         : String[015];  {'BUSY' response}
    NoDialtone   : String[015];  {'NO DIALTONE' response}
    Ring         : String[015];  {'RING' response}
    Ringing      : String[015];  {'RINGING' response}
    AttemptOn    : String[255];  {Force redial for this responses}
    AnswerOn     : String[255]   {Force answer for this responses}
  end;

  PHayesWordRec = ^THayesWordRec;
  THayesWordRec = record
    FlowControl   : Byte;
    Redial        : Byte;
    RingIndicator : Boolean;
    InitModem     : Boolean;
    BufSize       : Word
  end;

  TPortReg = record
    DAT,                     {+0}
    ICR,                     {+1}
    IIR,                     {+2}
    LCR,                     {+3}
    MCR,                     {+4}
    LSR,                     {+5}
    MSR,                     {+6}
    SCR: Byte;               {+7}
    Baud: Word
  end;

  TChannelInit = record
    Base         : Word;
    IRQ          : Byte;
    Params       : Byte;
    Vector       : Word;
    IncomingCall : Boolean;
    BaudRate     : Word;
    BaudRange    : LongRec;
    FrameSize    : LongRec;
    ModemFlags   : Word;
    Modem        : Byte;
    Line         : Byte;
    Protocol     : Byte;
    HayesWord    : THayesWordRec;
    HayesStr     : THayesStringRec;
    EmsiDat      : TEmsiDat
  end;

  TProtocolsSet  = Set of Byte;
  TProtocolsInit = record
  end;

Const
  ESO = 0;
  DZA = 1;
  ZAP = 2;
  ZMO = 3;
  JAN = 4;
  KER = 5;

  (*--- Offset to UART registers *)
  ofsDATA   = 0;
  ofsICR    = 1;
  ofsIIR    = 2; { Read }
  ofsFCR    = 2; { Write (UART above or equ 16550) }
  ofsLCR    = 3;
  ofsMCR    = 4;
  ofsLSR    = 5;
  ofsMSR    = 6;
  ofsDIVLSB = ofsDATA+0;
  ofsDIVMSB = ofsDATA+1;

  (*--- ICR Masks *)
  icNone      = $00;
  icReceive   = $01;
  icSend      = $02;
  icChangeLSR = $04;
  icChangeMSR = $08;

  (*--- IIR Masks *)
  iiNone      = $01;
  iiChangeMSR = $00;
  iiIsSend    = $02;
  iiIsReceive = $04;
  iiChangeLSR = $06;

  (*--- LCR Masks *)
  lc5bit      = $00;
  lc6bit      = $01;
  lc7bit      = $02;
  lc8bit      = $03;
  lcStop1     = $00;
  lcStop2     = $04;
  lcNoParity  = $00;
  lcEven      = $08;
  lcOdd       = $18;
  lcFixParity = $20;
  lcTxD       = $40;
  lcDLAB      = $80;
  lcNotDLAB   = $7F;

  (*--- MCR Masks *)
  mcDTR       = $01;
  mcRTS       = $02;
  mcOUT1      = $04;
  mcOUT2      = $08;
  mcSelfTest  = $10;

  (*--- LSR Masks *)
  lsReceiveByte = $01;
  lsLostByte    = $02;
  lsParityError = $04;
  lsSynError    = $08;
  lsFoundBreak  = $10;
  lsSendReady   = $20;
  lsSendByte    = $40;
  lsTimeOut     = $80;

  (*--- MSR Masks *)
  msChangeCTS   = $01;
  msChangeDSR   = $02;
  msChangeRI    = $04;
  msChangeCD    = $08;
  msCTS         = $10;
  msDSR         = $20;
  msRI          = $40;
  msCD          = $80;
  msChangePower = msChangeCTS + msChangeDSR;
  msPower       = msCTS + msDSR;

  (*--- Port Speed deviders *)
  bps110    = $0600;
  bps150    = $0300;
  bps300    = $0180;
  bps600    = $00C0;
  bps1200   = $0060;
  bps2400   = $0030;
  bps4800   = $0018;
  bps5236   = $0016;
  bps7200   = $0010;
  bps9600   = $000C;
  bps10472  = $000B;
  bps14400  = $0008;
  bps19200  = $0006;
  bps28800  = $0004;
  bps38400  = $0003;
  bps57600  = $0002;
  bps115200 = $0001;
  MaxBpsIdx = 16;
  BpsArray  : Array[0..MaxBpsIdx] of Word = (
    bps110,   bps150,   bps300,   bps600,   bps1200,  bps2400,
    bps4800,  bps5236,  bps7200,  bps9600,  bps10472, bps14400,
    bps19200, bps28800, bps38400, bps57600, bps115200
  );

  (*--- Error codes *)
  erOk                  = 0;
  erNone                = erOk;
  erTargetIsDead        = 1;    {                  }
  erGoatScuffle         = 2;    { 筠             }
  erUnexpectedBreak     = 3;    {  ࠧꥤ      }
  erSackSank            = 4;    { ९  ਥ    }
  erPatienceIsExhausted = 5;    {  믮     }
  erBadWire             = 6;    {  ४砥 ࠪ  }
  erDontPower           = 7;    {  ⠭     }
  erLineIsDead          = 8;    {                    }
  erLongHoots           = 9;    {    㡪       }
  erConstHoot           = 10;   { 訡                     }
  erNoHoot              = 11;   {   祭           }
  erBadInit             = 12;   {   樠     }
  erWaitEvent           = 13;   {                               }
  erLineError           = 255;  { 訡 LSR                    }
  erNotReady            = 254;  {   ⮢                }
  erNoProtocol          = 253;  {  । ⮪         }
  erUserBreak           = 252;  {                               }

  (*--- TCI Work flags *)
  wfNone            = $0000;
  wfSetTime         = $0001; {(P)ppp ⥬ p            --}
  wfAltConnect      = $0002; {(M)뢠 ᮡ p       ++}
  wfWaitBeep        = $0004; {(M) 㤮 p p ⥫䮭  W    ++}
  wfFlatcherCheck   = $0008; {(P)p p樨  p        ++}
  wfCrc32Check      = $0010; {(P)p p樨  CRC32           --}
  wfEnableAdjust    = $0020; {(M)pp襭 樨 ᪮p p       ++}
  wfEnableFrmChange = $0040; {(P)pp襭  pp p        --}
  wfCheckBusy       = $0080; {(M)p   CD p p祩      +-}
  wfMasterMode      = $0100; {+wfMasterMode, -wfFixedMode = M->,S<- (Normal) }
                             {-wfMasterMode, -wfFixedMode = S->,M<- (Reverse)}
                             {+wfMasterMode, +wfFixedMode = M->,M<- (Naster) }
  wfFixedMode       = $0200; {-wfMasterMode, +wfFixedMode = S->,S<- (Slave)  }
  wfPhoneLine       = $0400; {(M)⥫䮭                           ++}
  wfOldSupport      = $0800; {p   pp樨 p             }
  wfAnswerMode      = $1000; {(M) Hayes  뤥          }

  {* -- Modem operation *}
  moNothing            =   0; { ᢮                                }
  moInit               =   1; {p   易   ꥪ⮢       }
  moDiscoverEntryCall  =   2; {⠭ ᮥ  室饬 맮    }
  moDialing            =   3; {Hp p                                  }
  moConnect            =   4; {⠭ ᮥ                       }
  moDisconnect         =   5; {ꥤ                                  }
  moExchange           =   6; {ਥ(।)                         }

  {* -- Frame flags *}
  ffNULL   = $00;
  ffSYN    = $16;
  ffSTB    = $01;
  ffETB    = $17;
  ffSYNSTB = $1601;
  ffSYNETB = $1617;
  {* -- Frame types *}
  ftDATA   =  $40;
  ftWAIT   =  $41;
  ftEOT    =  $42;
  ftRR     =  $43;
  ftRNR    =  $44;
  ftTIME   =  $45;
  ftREQ    =  $46;
  ftSEL    =  $47;
  ftDISC   =  $49;

  fcNone        = $00;
  fcHardWareDSR = $01;
  fcHardWareCTS = $02;
  fcSoftWare    = $04;

  MaxPort   = 15;    {MaxCount ports to one IRQ}
  FirstIRQ  = $02;   {First used IRQ           }
  {$IFNDEF PCXT}     {Last used IRQ            }
    LastIRQ = $0F;
  {$ELSE}
    LastIRQ = $07;
  {$ENDIF}

  hfNone  = $0000;
  hfLoop  = $0001;
  hfBreak = $0003;

Implementation
End.
