(* Advance Utilities Ŀ*)
(*                                                                        *)
(*        Turbo Pascal Version 7.0                                        *)
(*                                                                        *)
(*        Copyright (c) 1992,95                      ZIG & AVM SofWare    *)
(*                                                                        *)
(**)

unit AdvUtil;

interface

  Uses Objects, Dos, TimeType, Inlines;

type
  MaskStr = String[128];

const
  HexChar : array[0..$F] of Char ='0123456789ABCDEF';

  smDevice    = #01;
  smDir       = #02;
  smName      = #03;
  smExt       = #04;
  smPath      = smDevice + smDir;
  smFile      = smPath + smName;
  smFullPath  = smFile + smExt;

{ File split mode constant }
  fsDir     = $01;
  fsName    = $02;
  fsExt     = $04;

function FileExists(const FileName: FNameStr) : boolean;
function EraseFile(const FileName: FNameStr) : boolean;
function FSplitMask(const aName : PathStr; const aMask : MaskStr) : PathStr;
function FileSplit(const Path : PathStr; Mode : byte) : string;
function ValidDrive(D : char) : boolean;
function MakeDir(const DirName : PathStr) : boolean;
function CheckDir(DirName : PathStr) : boolean;

function CompressPath(Path : string; MaxLen : byte) : string;
procedure TrailSpaces(var S : string);
function CenterStr(const Src : string; Len : byte) : string;
function IntervalStr(T : longint) : string;
function StrPos(const Substr, Source : string; StartPos : byte) : byte;
function UpCaseStr(S : string) : string;
function Tab2Space(Source : string) : string;

function Sign(I : integer) : integer;
function LSign(L : longint) : integer;
function RSign(X : real) : integer;

function IntStr(Value : longint) : string;
function ExpIntStr(Value : longint; Width : byte) : string;
function FixIntStr(Value : longint; Width : byte) : string;
function HexLongStr(W : LongInt) : string;
function HexWordStr(W : Word) : string;
function HexByteStr(W : Byte) : string;
function RealStr(Value : real; Width,Decimals : byte) : string;
function FloatStr(Value : extended; Width,Decimals : byte) : string;

function Equal(var Source, Dest; Size : word): boolean;
function CheckFill(var Bag; Size : word; Value : byte) : boolean;
procedure ExchangeStructs(var I, J; Size : Word);

function ArcSin(X : real) : real;
function ArcCos(X : real) : real;
function Power(X,Y : real) : real;

function DateFName(var T : TDateStruc) : FNameStr;
function CompareTime(var T,T1) : Integer;
function Date2Str(var D : TDateStruc; Delims : boolean) : string;
procedure IncreaseTime(var H,M,S : byte; Value : word);
function DosTimer : longint;

implementation

function FileSplit(const Path : PathStr; Mode : Byte) : String;
var
   D : DirStr;
   N : NameStr;
   E : ExtStr;
   S : String;
begin
  S:='';
  FSplit(Path, D, N, E);
  if (Mode and fsDir) <> 0 then S:=D;
  if (Mode and fsName) <> 0 then S:=S+N;
  if (Mode and fsExt) <> 0 then S:=S+E;
  FileSplit:=S;
end;

function FileExists(const FileName: FNameStr) : boolean;
  var
    DirInfo : SearchRec;
  begin
    FindFirst(FileName, AnyFile, DirInfo);
    FileExists := (DosError = 0) and (FileName <> '');
  end;  { FileExists }

function EraseFile(const FileName: FNameStr) : boolean;
  var F : file;
  begin
    {$I-}
    assign(F, FileName);
    erase(F);
    EraseFile := IOResult = 0;
  end;

function FSplitMask(const aName : PathStr; const aMask : MaskStr) : PathStr;
  var
    DP     : DirStr;
    N      : NameStr;
    E      : ExtStr;
    Dev    : String[2];
    i      : byte;
    BufStr : PathStr;
  begin
    BufStr := '';
    FSplit( aName, DP, N, E );
    if (length(DP) <> 0) and (DP[2] = ':') then
                  begin
        Dev := copy(DP,1,2);
        delete(DP,1,2);
      end
    else Dev := '';
    for i := 1 to length(aMask) do
      begin
        case aMask[i] of
          smDevice : BufStr := BufStr + Dev;
          smDir    : BufStr := BufStr + DP;
          smName   : BufStr := BufStr + N;
          smExt    : BufStr := BufStr + E;
          else
            BufStr := BufStr + aMask[i];
        end;
      end;
    FSplitMask := BufStr;
  end;

function ValidDrive(D : char) : boolean; assembler;
  asm
    mov   dl, D
    cmp   dl, $40
    jp    @Quit
    cmp   dl, $80
    jge   @Quit
    and   dl, $1F
    cmp   dl, $1B
    jge   @Quit
    mov   ax, $1C00
    int   21h
    cmp   al, $FF
    jne   @Exit
@Quit:
    xor   ax, ax
    jmp   @EndF
@Exit:
    mov   al, 1
@EndF:
  end;

function MakeDir(const DirName : PathStr) : boolean;
  begin
    {$I-}
    MkDir(DirName);
    {$I+}
    MakeDir := (IOResult = 0) and (DirName <> '');
  end;

function CheckDir(DirName : PathStr) : boolean;
  begin
    CheckDir := true;
    if DirName[length(DirName)] = '\' then dec(byte(DirName[0]));
    case length(DirName) of
      0 : exit;
      2 : if DirName[2] = ':' then
        begin
          CheckDir := ValidDrive(DirName[1]);
          exit;
        end;
    end;
    CheckDir := FileExists(DirName) or MakeDir(DirName);
  end;

function CompressPath(Path : string; MaxLen : byte) : string;
  var
    P1,P2 : byte;
    InsFlag : boolean;
  begin
    if length(Path) > MaxLen then
      begin
        p1 := pos('\', Path);
        InsFlag := true;
        if p1 <> 0 then
          repeat
            p2 := StrPos('\', Path, p1+1);
            if p2 = 0 then break;
            delete(Path, p1, p2-p1);
            if InsFlag then
              begin
                insert('\..', Path, p1);
                InsFlag := false;
                inc(p1, 3);
              end;
          until length(Path) <= MaxLen;
      end;
    CompressPath := Path;
  end;

procedure TrailSpaces(var S : string);
  var i : integer;
  begin
    repeat
      i := pos(' ', s);
      if i = 0 then break;
      delete(s, i, 1);
    until false;
  end;

function CenterStr(const Src : string; Len : byte) : string;
  var Dst : string;
  begin
    if Len < length(Src) then CenterStr := copy(Src, 1, Len)
    else
      begin
        Dst[0] := chr((Len - length(Src)) shr 1);
        FillChar(Dst[1], ord(Dst[0]), ' ');
        Dst := Dst + Src;
        FillChar(Dst[length(Dst)+1], Len - length(Dst), ' ');
        Dst[0] := chr(Len);
        CenterStr := Dst;
      end;
  end;

function IntervalStr(T : longint) : string;
  var S : string[40];
  begin
    if T div 86400 <> 0 then S := IntStr( T div 86400 ) + '  '
    else S := '';
    T := T mod 86400;
    S := S + ExpIntStr( T div 3600, 2 ) + '  ';
    T := T mod 3600;
    S := S + ExpIntStr( T div 60, 2 ) + '  ' +
              ExpIntStr( T mod 60, 2 ) + ' ᥪ';
    IntervalStr := S;
  end;

function StrPos(const Substr, Source : string; StartPos : byte) : byte;
  var p : byte;
  begin
    p := pos(SubStr, copy(Source, StartPos, length(Source)-StartPos+1));
    if p <> 0 then inc(p, StartPos-1);
    StrPos := p;
  end;

function UpCaseStr(S : string) : string; assembler;
asm
        jmp @S
@U:     db 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,11,12,13,14,15
        db 16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
        db ' ','!','"','#','$','%','&',39,'(',')','*','+',',','-','.','/'
        db '0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','?'
        db '@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O'
        db 'P','Q','R','S','T','U','V','W','X','Y','Z','[','\',']','^','_'
        db '`','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O'
        db 'P','Q','R','S','T','U','V','W','X','Y','Z','{','|','}','~',''
        db '','','','','','','','','','','','','','','',''
        db '','','','','','','','','','','','','','','',''
        db '','','','','','','','','','','','','','','',''
        db $B0,$B1,$B2,$B3,$B4,$B5,$B6,$B7,$B8,$B9,$BA,$BB,$BC,$BD,$BE,$BF
        db $C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF
        db $D0,$D1,$D2,$D3,$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF
        db '','','','','','','','','','','','','','','',''
        db $F0,$F1,$F2,$F3,$F4,$F5,$F6,$F7,$F8,$F9,$FA,$FB,$FC,$FD,$FE,$FF
@S:     push ds
        lds si, S
        les di, @Result
        cld
        mov cl,[si]
        movsb
        xor ch,ch
        jcxz @@Q
        mov bx, offset @U
@@1:    lodsb
        segcs xlat
        stosb
        loop @@1
@@Q:    pop ds
end;

function Tab2Space(Source : String) : String; assembler;
asm
        push ds
        lds si,Source
        les di,@Result
        inc di
        cld
        lodsb
        mov cl,al
        xor ch,ch
        xor dx,dx { dx = number of transferred chars }
        jcxz @@1
@@4:    lodsb
        cmp al,9
        jne @@2
        mov al,20h
        test dx,7
        xchg ax,dx
        jz @@2
        dec si     { simulate new tab character }
        inc cx
@@2:    stosb
        inc dx
        cmp dx,255 { length checking }
        jge @@1
        loop @@4
@@1:    pop ds
        sub di,dx  { store result string length }
        dec di
        mov es:[di],dl
end;

function Sign(I : integer) : integer; assembler;
  asm
    xor ax,ax
    mov bx,i
    or  bx,bx
    jz  @@2
    js  @@1
    inc ax
    jmp @@2
@@1:dec ax
@@2:
  end;

function LSign(L : longint) : integer; assembler;
  asm
    xor ax,ax
    mov bx,word ptr L+2
    or  bx,bx
    jz  @@2
    js  @@1
    inc ax
    jmp @@3
@@1:dec ax
    jmp @@3
@@2:mov bx,L.word
    or  bx,bx
    jz  @@3
    inc ax
@@3:
  end;

function RSign(X : real) : integer; assembler;
  asm
    xor ax,ax
    mov bx,x.word
    or  bx,bx
    jz  @@2
    js  @@1
    inc ax
    jmp @@2
@@1:dec ax
@@2:
  end;

function IntStr(Value : longint) : string;
  var S : string[19];
  begin
    Str(Value, S);
    IntStr := S;
  end;

function ExpIntStr(Value : longint; Width : byte) : string;
  var S : string[19];
  begin
    Str(Value:Width, S);
    asm
      lea   si, S
      les   di, @Result
      xor   ch, ch
      segss lodsb
      stosb
      mov   cl, al
      jcxz  @E
@L:   segss lodsb
      cmp   al, ' '
      jne   @St
      mov   al, '0'
@St:  stosb
      loop  @L
@E:
    end;
  end;

function FixIntStr(Value : longint; Width : byte) : string;
  var S : string[19];
  begin
    Str(Value:Width, S);
    FixIntStr := S;
  end;

function HexLongStr(W : LongInt) : string;
  begin
    HexLongStr := HexWordStr(LongRec(W).Hi)+HexWordStr(LongRec(W).Lo);
  end;

function HexWordStr(W : word) : string;
  begin
    HexWordStr[0] := #4;
    HexWordStr[1] := HexChar[Hi(w) shr $4];
    HexWordStr[2] := HexChar[Hi(w) and $F];
    HexWordStr[3] := HexChar[Lo(w) shr $4];
    HexWordStr[4] := HexChar[Lo(w) and $F];
  end;

function HexByteStr(W : byte) : string;
  begin
    HexByteStr[0] := #2;
    HexByteStr[1] := HexChar[W shr $4];
    HexByteStr[2] := HexChar[W and $F];
  end;

function RealStr(Value : real; Width,Decimals : byte) : string;
  var S : string[23];
      Ex,Er : integer;
      P,Z,Pf : integer;
  procedure ExpMode;
    begin
      z := p+1;
      if s[z] = '-' then inc(z)
      else delete(s, z, 1);
      while (s[z] = '0') and (z < length(s)) do delete(s, z, 1);
      z := length(s) - width;
      if z > 0 then delete( s, p - z, z );
    end;
  begin
    str(Value, S);
    pf := ord(s[1] = '-');
    if pf = 0 then delete(s, 1, 1);
    p := pos('E', S);
    val(copy(s, p+1, length(s)-p), Ex, Er);
    if ex >= 0 then
      if ex+pf >= width then ExpMode
      else
        begin
          z := ex+2+pf;
          if z > p then fillchar(s[p], z-p+1, '0');
          s[0] := char(p-1);
          p := pos('.', s);
          if p < z then
            begin
              delete(s, p, 1);
              insert('.', s, z);
            end;
          if Decimals = 0 then s[0] := char(z-1)
          else
            begin
              if length(s) > width then s[0] := char(width);
              if length(s) > decimals + z then s[0] := char(z + decimals);
            end;
        end
      else
        begin
          ex := -ex;
          z := ex+1+pf;
          if decimals = 0 then S := '0' else
          if (width <= z) or ((width - z < decimals) and (ex > 3)) then ExpMode
          else
            begin
              s[0] := char(p-1);
              p := pos('.', s);
              delete(s, p, 1);
              move(s[p-1], s[z+1], width-z);
              fillchar(s[p-1], ex+1, '0');
              s[p] := '.';
              if length(s) > width then s[0] := char(width);
              if length(s) > decimals + z then s[0] := char(z + decimals);
            end;
        end;
    RealStr := S;
  end;

function FloatStr(Value : extended; Width, Decimals : byte) : string;
  var S,S1 : string[23];
      Ex,E : integer;
      pf : byte;
  procedure ExpMode;
    var i : integer;
    begin
      e := 1;
      for i := 2 to 5 do if s1[i] = '0' then inc(e) else break;
      inc(Width, e - pf + ord(ex > 0));
      str(Value:Width, S);
      i := length(s)-4;
      if ex < 0 then
        begin
          inc(i); dec(e);
        end;
      delete(s, i, e);
      if pf = 0 then delete(s, 1, 1);
      if length(s) > width then
        begin
          i := pos('E', S)-1;
          repeat
            delete(s, i, 1);
            dec(i);
          until (length(s) = width) or (i = 0);
        end;
    end;
  begin
    str(Value, S); S1 := copy(s, 19, 5); val(S1, Ex, E);
    pf := ord(s[1] = '-');
    repeat
    if Ex + pf >= width then begin ExpMode; break; end
    else
      begin
        if Ex < 0 then
          begin
            e := minint(Decimals-ex-1, Width-pf-2);
            if e+ex+2 < decimals then begin ExpMode; break; end;
          end
        else e := maxint(0, minint(Decimals, Width-pf-2-ex));
        Decimals := e;
        str(Value:Width:decimals, S);
      end;
    until true;
    while s[1] = ' ' do delete(s, 1, 1);
    FloatStr := s;
  end;

function Equal(var Source, Dest; Size : word): boolean; assembler;
  asm
    cld
    xor   ax, ax
    mov   bx, ds
    lds   si, Source
    les   di, Dest
    mov   cx, Size
    test  cx, $01
    jz    @@1
    repe  cmpsb
    jmp   @@2
  @@1:
    shr   cx,1
    repe  cmpsw
  @@2:
    jne   @@3
    inc   ax
  @@3:
    mov   ds, bx
  end;

function CheckFill(var Bag; Size : word; Value : byte) : boolean; assembler;
  asm
    cld
    xor   ax, ax
    les   di, Bag
    mov   cx, Size
    mov   al, Value
    repe  scasb
    jne   @@1
    inc   ax
  @@1:
  end;

function ArcSin(X : real) : real;
  var x2 : real;
  begin
    x2 := sqr(x);
    if x2 < 1 then ArcSin := ArcTan( x / sqrt(1-x2) )
    else ArcSin := RSign(x) * Pi/2;
  end;

function ArcCos(X : real) : real;
  begin
    ArcCos := Pi/2 - ArcSin(x);
  end;

function Power(X,Y : real) : real;
  begin
    Power := exp( ln(x) * y );
  end;

procedure ExchangeStructs(var I, J; Size : Word); assembler;
asm
    cld
    mov dx,ds
    mov cx, Size
    lds si, J
    les di, I
    shr cx,1
    jcxz @1
    pushf
@2:
    mov bx,si
    mov ax,es:[di]
    movsw
    mov [bx],ax
    loop @2
    popf
@1:
    jnc @3
    mov al,[si]
    xchg al,es:[di]
    mov [si],al
@3:
    mov ds,dx
end;

function DateFName(var T : TDateStruc) : FNameStr;
  begin
    DateFName := ExpIntStr(T.Day, 2) + ExpIntStr(T.Month, 2) +
                 + ExpIntStr(T.Year, 4);
  end;

function Date2Str(var D : TDateStruc; Delims : boolean) : string;
  var S : string[10];
  begin
    S := ExpIntStr(D.Day, 2) + ExpIntStr(D.Month, 2) + ExpIntStr(D.Year, 4);
    if Delims then
      begin
        insert('-', S, 3);
        insert('-', S, 6);
      end;
    Date2Str := S;
  end;

procedure IncreaseTime(var H,M,S : byte; Value : word); assembler;
  asm
    xor   ax, ax
    les   di, S
    mov   al, es:[di]
    add   ax, Value
    cmp   ax, 59
    ja    @SecOvf
    mov   es:[di],  al
    jmp   @Quit
  @SecOvf:
    xor   dx, dx
    mov   bx, 60
    div   bx
    mov   es:[di],  dl
    les   di, M
    add   al, es:[di]
    adc   ah, 0
    cmp   ax, 59
    ja    @MinOvf
    mov   es:[di],  al
    jmp   @Quit
  @MinOvf:
    xor   dx, dx
    mov   bx, 60
    div   bx
    mov   es:[di],  dl
    les   di, H
    add   al, es:[di]
    adc   ah, 0
    cmp   ax, 23
    ja    @HourOvf
    mov   es:[di],  al
    jmp   @Quit
  @HourOvf:
    xor   dx, dx
    mov   bx, 24
    div   bx
    mov   es:[di],  dl
  @Quit:
  end;

function DosTimer : longint; assembler;
  asm
    mov   ax, Seg0040
    mov   es, ax
    cli
    mov   ax, es:[$6C]
    mov   dx, es:[$6E]
    sti
  end;

function CompareTime(var T,T1) : integer; assembler;
asm
     xor    ax, ax
     les    di, T
     mov    dh, es:[di].TTimeStruc.Hour
     mov    dl, es:[di].TTimeStruc.Min
     mov    cl, es:[di].TTimeStruc.Sec
     les    di, T1
     cmp    dh, es:[di].TTimeStruc.Hour
     jb     @1 {<}
     ja     @2 {>}
     cmp    dl, es:[di].TTimeStruc.Min
     jb     @1
     ja     @2
     cmp    cl, es:[di].TTimeStruc.Sec
     jb     @1
     je     @3
@2:  inc    ax
     jmp    @3
@1:  dec    ax
@3:
end;

end.
